@@ -24,10 +24,9 @@
  * the rights to redistribute these changes.
  */
 
-#if MACH_TTD
+#include "mach_ttd.h"
 
-#include <sys/types.h>
-#include <kern/printf.h>
+#if MACH_TTD
 
 #include <mach/machine/eflags.h>
 
@@ -36,6 +35,7 @@
 #include <mach/thread_status.h>
 #include <mach/vm_param.h>
 #include <i386/seg.h>
+#include <sys/types.h>
 
 #include <ttd/ttd_types.h>
 #include <ttd/ttd_stub.h>
