@@ -52,6 +52,8 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #ifndef	_I386_PIC_H_
 #define	_I386_PIC_H_
 
+#include <platforms.h>
+
 #define NINTR	0x10
 #define	NPICS	0x02
 
@@ -59,12 +61,19 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ** The following are definitions used to locate the PICs in the system
 */
 
-#if	defined(AT386)
+#if	defined(AT386) || defined(PS2)
 #define ADDR_PIC_BASE		0x20
 #define OFF_ICW			0x00
 #define OFF_OCW			0x01
 #define SIZE_PIC		0x80
-#endif	/* defined(AT386) */
+#endif	/* defined(AT386) || defined(PS2) */
+
+#ifdef	iPSC386
+#define ADDR_PIC_BASE		0xC0
+#define OFF_ICW			0x00
+#define OFF_OCW			0x02
+#define SIZE_PIC		0x04
+#endif	/* iPSC386 */
 
 #define PIC_MASTER_ICW		(ADDR_PIC_BASE + OFF_ICW)
 #define PIC_MASTER_OCW		(ADDR_PIC_BASE + OFF_OCW)
@@ -95,10 +104,15 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 **	ICW2
 */
 
-#if	defined(AT386)
+#if	defined(AT386) || defined(PS2)
 #define	PICM_VECTBASE		0x40
 #define PICS_VECTBASE		PICM_VECTBASE + 0x08
-#endif	/* defined(AT386) */
+#endif	/* defined(AT386) || defined(PS2) */
+
+#ifdef	iPSC386
+#define	PICM_VECTBASE		0x40
+#define PICS_VECTBASE		PICM_VECTBASE + 0x08
+#endif	/* iPSC386 */
 
 /*
 **	ICW3
@@ -130,6 +144,10 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define SNF_MODE_DIS		0x00
 #define BUFFERD_MODE		0x08
 #define NONBUFD_MODE		0x00
+#if	iPSC386
+#define I_AM_A_SLAVE		0x00
+#define I_AM_A_MASTR		0x04
+#endif	/* iPSC386 */
 #define AUTO_EOI_MOD		0x02
 #define NRML_EOI_MOD		0x00
 #define I8086_EMM_MOD		0x01
@@ -176,13 +194,4 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define READ_IR_ONRD		0x00
 #define READ_IS_ONRD		0x01
 
-#ifndef __ASSEMBLER__
-extern void form_pic_mask (void);
-extern void picinit (void);
-extern int curr_pic_mask;
-extern int pic_mask[];
-extern void prtnull(int unit);
-extern void intnull(int unit);
-#endif /* __ASSEMBLER__ */
-
 #endif	/* _I386_PIC_H_ */
