@@ -32,10 +32,6 @@
 
 #define PIC_INT_BASE 0x40
 
-#include <i386/idt-gen.h>
-
-#ifndef __ASSEMBLER__
-extern void idt_init (void);
-#endif /* __ASSEMBLER__ */
+#include "idt-gen.h"
 
 #endif /* _I386AT_IDT_ */
