@@ -31,53 +31,64 @@
  *	Thread management primitives implementation.
  */
 
-#include <kern/printf.h>
+#include <cpus.h>
+#include <hw_footprint.h>
+#include <mach_host.h>
+#include <mach_fixpri.h>
+#include <mach_pcsample.h>
+#include <simple_clock.h>
+#include <mach_debug.h>
+#include <net_atm.h>
+
 #include <mach/std_types.h>
 #include <mach/policy.h>
 #include <mach/thread_info.h>
 #include <mach/thread_special_ports.h>
 #include <mach/thread_status.h>
 #include <mach/time_value.h>
-#include <machine/vm_param.h>
+#include "vm_param.h"
 #include <kern/ast.h>
 #include <kern/counters.h>
-#include <kern/debug.h>
-#include <kern/eventcount.h>
-#include <kern/ipc_mig.h>
 #include <kern/ipc_tt.h>
+#include <kern/mach_param.h>
 #include <kern/processor.h>
 #include <kern/queue.h>
 #include <kern/sched.h>
 #include <kern/sched_prim.h>
-#include <kern/syscall_subr.h>
 #include <kern/thread.h>
 #include <kern/thread_swap.h>
 #include <kern/host.h>
-#include <kern/kalloc.h>
-#include <kern/slab.h>
-#include <kern/mach_clock.h>
+#include <kern/zalloc.h>
 #include <vm/vm_kern.h>
-#include <vm/vm_user.h>
 #include <ipc/ipc_kmsg.h>
 #include <ipc/ipc_port.h>
 #include <ipc/mach_msg.h>
-#include <ipc/mach_port.h>
 #include <machine/machspl.h>		/* for splsched */
-#include <machine/pcb.h>
 #include <machine/thread.h>		/* for MACHINE_STACK */
 
+#if	NET_ATM
+#include <chips/nw_mk.h>
+#endif
+
 thread_t active_threads[NCPUS];
 vm_offset_t active_stacks[NCPUS];
 
-struct kmem_cache thread_cache;
+struct zone *thread_zone;
 
 queue_head_t		reaper_queue;
 decl_simple_lock_data(,	reaper_lock)
 
+extern int		tick;
+
+extern void		pcb_module_init(void);
+
 /* private */
 struct thread	thread_template;
 
 #if	MACH_DEBUG
+void stack_init(vm_offset_t stack);	/* forward */
+void stack_finalize(vm_offset_t stack);	/* forward */
+
 #define	STACK_MARKER	0xdeadbeefU
 boolean_t		stack_check_usage = FALSE;
 decl_simple_lock_data(,	stack_usage_lock)
@@ -146,7 +157,7 @@ boolean_t stack_alloc_try(
 	thread_t	thread,
 	void		(*resume)(thread_t))
 {
-	vm_offset_t stack;
+	register vm_offset_t stack;
 
 	stack_lock();
 	stack = stack_free_list;
@@ -205,7 +216,7 @@ void stack_alloc(
 		 *	addresses of a stack given an address in the middle.
 		 */
 
-		if (kmem_alloc_aligned(kmem_map, &stack, KERNEL_STACK_SIZE)
+		if (kmem_alloc_aligned(kernel_map, &stack, KERNEL_STACK_SIZE)
 							!= KERN_SUCCESS)
 			panic("stack_alloc");
 
@@ -227,7 +238,7 @@ void stack_alloc(
 void stack_free(
 	thread_t thread)
 {
-	vm_offset_t stack;
+	register vm_offset_t stack;
 
 	stack = stack_detach(thread);
 
@@ -250,7 +261,7 @@ void stack_free(
 
 void stack_collect(void)
 {
-	vm_offset_t stack;
+	register vm_offset_t stack;
 	spl_t s;
 
 	s = splsched();
@@ -265,7 +276,7 @@ void stack_collect(void)
 #if	MACH_DEBUG
 		stack_finalize(stack);
 #endif	/* MACH_DEBUG */
-		kmem_free(kmem_map, stack, KERNEL_STACK_SIZE);
+		kmem_free(kernel_map, stack, KERNEL_STACK_SIZE);
 
 		s = splsched();
 		stack_lock();
@@ -282,7 +293,7 @@ void stack_collect(void)
  */
 
 void stack_privilege(
-	thread_t thread)
+	register thread_t thread)
 {
 	/*
 	 *	This implementation only works for the current thread.
@@ -297,8 +308,11 @@ void stack_privilege(
 
 void thread_init(void)
 {
-	kmem_cache_init(&thread_cache, "thread", sizeof(struct thread), 0,
-			NULL, NULL, NULL, 0);
+	thread_zone = zinit(
+			sizeof(struct thread),
+			THREAD_MAX * sizeof(struct thread),
+			THREAD_CHUNK * sizeof(struct thread),
+			0, "threads");
 
 	/*
 	 *	Fill in a template thread for fast initialization.
@@ -395,11 +409,11 @@ void thread_init(void)
 }
 
 kern_return_t thread_create(
-	task_t	parent_task,
+	register task_t	parent_task,
 	thread_t	*child_thread)		/* OUT */
 {
-	thread_t	new_thread;
-	processor_set_t	pset;
+	register thread_t	new_thread;
+	register processor_set_t	pset;
 
 	if (parent_task == TASK_NULL)
 		return KERN_INVALID_ARGUMENT;
@@ -408,7 +422,7 @@ kern_return_t thread_create(
 	 *	Allocate a thread and initialize static fields
 	 */
 
-	new_thread = (thread_t) kmem_cache_alloc(&thread_cache);
+	new_thread = (thread_t) zalloc(thread_zone);
 
 	if (new_thread == THREAD_NULL)
 		return KERN_RESOURCE_SHORTAGE;
@@ -434,6 +448,10 @@ kern_return_t thread_create(
 
 	ipc_thread_init(new_thread);
 
+#if	NET_ATM
+	new_thread->nw_ep_waited = 0;
+#endif
+
 	/*
 	 *	Find the processor set for the parent task.
 	 */
@@ -530,6 +548,7 @@ kern_return_t thread_create(
 #endif	/* HW_FOOTPRINT */
 
 #if	MACH_PCSAMPLE
+	new_thread->pc_sample.buffer = 0;
 	new_thread->pc_sample.seqno = 0;
 	new_thread->pc_sample.sampletypes = 0;
 #endif	/* MACH_PCSAMPLE */
@@ -572,11 +591,11 @@ kern_return_t thread_create(
 unsigned int thread_deallocate_stack = 0;
 
 void thread_deallocate(
-	thread_t	thread)
+	register thread_t	thread)
 {
 	spl_t		s;
-	task_t		task;
-	processor_set_t	pset;
+	register task_t	task;
+	register processor_set_t	pset;
 
 	time_value_t	user_time, system_time;
 
@@ -692,7 +711,7 @@ void thread_deallocate(
 	 *	Clean up any machine-dependent resources.
 	 */
 	if ((thread->state & TH_SWAPPED) == 0) {
-		splsched();
+		spl_t _s_ = splsched();
 		stack_free(thread);
 		(void) splx(s);
 		thread_deallocate_stack++;
@@ -703,11 +722,11 @@ void thread_deallocate(
 	evc_notify_abort(thread);
 
 	pcb_terminate(thread);
-	kmem_cache_free(&thread_cache, (vm_offset_t) thread);
+	zfree(thread_zone, (vm_offset_t) thread);
 }
 
 void thread_reference(
-	thread_t	thread)
+	register thread_t	thread)
 {
 	spl_t		s;
 
@@ -741,10 +760,10 @@ void thread_reference(
  *	since it needs a kernel stack to execute.)
  */
 kern_return_t thread_terminate(
-	thread_t	thread)
+	register thread_t	thread)
 {
-	thread_t		cur_thread = current_thread();
-	task_t			cur_task;
+	register thread_t	cur_thread = current_thread();
+	register task_t		cur_task;
 	spl_t			s;
 
 	if (thread == THREAD_NULL)
@@ -843,32 +862,13 @@ kern_return_t thread_terminate(
 	 *	reference to itself.
 	 */
 	ipc_thread_terminate(thread);
+#if	NET_ATM
+	mk_waited_collect(thread);
+#endif
 	thread_deallocate(thread);
 	return KERN_SUCCESS;
 }
 
-kern_return_t thread_terminate_release(
-	thread_t thread,
-	task_t task,
-	mach_port_t thread_name,
-	mach_port_t reply_port,
-	vm_offset_t address,
-	vm_size_t size)
-{
-	if (task == NULL)
-		return KERN_INVALID_ARGUMENT;
-
-	mach_port_deallocate(task->itk_space, thread_name);
-
-	if (reply_port != MACH_PORT_NULL)
-		mach_port_destroy(task->itk_space, reply_port);
-
-	if ((address != 0) || (size != 0))
-		vm_deallocate(task->map, address, size);
-
-	return thread_terminate(thread);
-}
-
 /*
  *	thread_force_terminate:
  *
@@ -878,9 +878,9 @@ kern_return_t thread_terminate_release(
  */
 void
 thread_force_terminate(
-	thread_t	thread)
+	register thread_t	thread)
 {
-	boolean_t	deallocate_here;
+	boolean_t	deallocate_here = FALSE;
 	spl_t s;
 
 	ipc_thread_disable(thread);
@@ -903,6 +903,9 @@ thread_force_terminate(
 
 	(void) thread_halt(thread, TRUE);
 	ipc_thread_terminate(thread);
+#if	NET_ATM
+	mk_waited_collect(thread);
+#endif
 
 #if	MACH_HOST
 	thread_unfreeze(thread);
@@ -920,11 +923,11 @@ thread_force_terminate(
  *
  */
 kern_return_t thread_halt(
-	thread_t	thread,
+	register thread_t	thread,
 	boolean_t		must_halt)
 {
-	thread_t	cur_thread = current_thread();
-	kern_return_t	ret;
+	register thread_t	cur_thread = current_thread();
+	register kern_return_t	ret;
 	spl_t	s;
 
 	if (thread == cur_thread)
@@ -968,7 +971,7 @@ kern_return_t thread_halt(
 		 *	operation can never cause a deadlock.)
 		 */
 		if (cur_thread->ast & AST_HALT) {
-			thread_wakeup_with_result(TH_EV_WAKE_ACTIVE(cur_thread),
+			thread_wakeup_with_result((event_t)&cur_thread->wake_active,
 				THREAD_INTERRUPTED);
 			thread_unlock(thread);
 			thread_unlock(cur_thread);
@@ -1006,7 +1009,7 @@ kern_return_t thread_halt(
 	 */
 	while ((thread->ast & AST_HALT) && (!(thread->state & TH_HALTED))) {
 		thread->wake_active = TRUE;
-		thread_sleep(TH_EV_WAKE_ACTIVE(thread),
+		thread_sleep((event_t) &thread->wake_active,
 			simple_lock_addr(thread->lock), TRUE);
 
 		if (thread->state & TH_HALTED) {
@@ -1045,7 +1048,7 @@ kern_return_t thread_halt(
 			s = splsched();
 			thread_lock(thread);
 			thread_ast_clear(thread, AST_HALT);
-			thread_wakeup_with_result(TH_EV_WAKE_ACTIVE(thread),
+			thread_wakeup_with_result((event_t)&thread->wake_active,
 				THREAD_INTERRUPTED);
 			thread_unlock(thread);
 			(void) splx(s);
@@ -1123,7 +1126,7 @@ kern_return_t thread_halt(
 	}
 }
 
-void __attribute__((noreturn)) walking_zombie(void)
+void	walking_zombie(void)
 {
 	panic("the zombie walks!");
 }
@@ -1134,7 +1137,7 @@ void __attribute__((noreturn)) walking_zombie(void)
  */
 void	thread_halt_self(void)
 {
-	thread_t	thread = current_thread();
+	register thread_t	thread = current_thread();
 	spl_t	s;
 
 	if (thread->ast & AST_TERMINATE) {
@@ -1144,12 +1147,15 @@ void	thread_halt_self(void)
 		 *	reaper thread.
 		 */
 		ipc_thread_terminate(thread);
+#if	NET_ATM
+		mk_waited_collect(thread);
+#endif
 
 		thread_hold(thread);
 
 		s = splsched();
 		simple_lock(&reaper_lock);
-		enqueue_tail(&reaper_queue, &(thread->links));
+		enqueue_tail(&reaper_queue, (queue_entry_t) thread);
 		simple_unlock(&reaper_lock);
 
 		thread_lock(thread);
@@ -1188,7 +1194,7 @@ void	thread_halt_self(void)
  *	suspends is maintained.
  */
 void thread_hold(
-	thread_t	thread)
+	register thread_t	thread)
 {
 	spl_t			s;
 
@@ -1210,11 +1216,11 @@ void thread_hold(
  */
 kern_return_t
 thread_dowait(
-	thread_t		thread,
+	register thread_t	thread,
 	boolean_t		must_halt)
 {
-	boolean_t		need_wakeup;
-	kern_return_t		ret = KERN_SUCCESS;
+	register boolean_t	need_wakeup;
+	register kern_return_t	ret = KERN_SUCCESS;
 	spl_t			s;
 
 	if (thread == current_thread())
@@ -1284,7 +1290,7 @@ thread_dowait(
 		     *	Check for failure if interrupted.
 		     */
 		    thread->wake_active = TRUE;
-		    thread_sleep(TH_EV_WAKE_ACTIVE(thread),
+		    thread_sleep((event_t) &thread->wake_active,
 				simple_lock_addr(thread->lock), TRUE);
 		    thread_lock(thread);
 		    if ((current_thread()->wait_result != THREAD_AWAKENED) &&
@@ -1308,13 +1314,13 @@ thread_dowait(
 	(void) splx(s);
 
 	if (need_wakeup)
-	    thread_wakeup(TH_EV_WAKE_ACTIVE(thread));
+	    thread_wakeup((event_t) &thread->wake_active);
 
 	return ret;
 }
 
 void thread_release(
-	thread_t	thread)
+	register thread_t	thread)
 {
 	spl_t			s;
 
@@ -1333,9 +1339,9 @@ void thread_release(
 }
 
 kern_return_t thread_suspend(
-	thread_t	thread)
+	register thread_t	thread)
 {
-	boolean_t		hold;
+	register boolean_t	hold;
 	spl_t			spl;
 
 	if (thread == THREAD_NULL)
@@ -1344,13 +1350,6 @@ kern_return_t thread_suspend(
 	hold = FALSE;
 	spl = splsched();
 	thread_lock(thread);
-	/* Wait for thread to get interruptible */
-	while (thread->state & TH_UNINT) {
-		assert_wait(TH_EV_STATE(thread), TRUE);
-		thread_unlock(thread);
-		thread_block(NULL);
-		thread_lock(thread);
-	}
 	if (thread->user_stop_count++ == 0) {
 		hold = TRUE;
 		thread->suspend_count++;
@@ -1379,9 +1378,9 @@ kern_return_t thread_suspend(
 
 
 kern_return_t thread_resume(
-	thread_t	thread)
+	register thread_t	thread)
 {
-	kern_return_t		ret;
+	register kern_return_t	ret;
 	spl_t			s;
 
 	if (thread == THREAD_NULL)
@@ -1417,7 +1416,7 @@ kern_return_t thread_resume(
  *	Return thread's machine-dependent state.
  */
 kern_return_t thread_get_state(
-	thread_t		thread,
+	register thread_t	thread,
 	int			flavor,
 	thread_state_t		old_state,	/* pointer to OUT array */
 	natural_t		*old_state_count)	/*IN/OUT*/
@@ -1441,7 +1440,7 @@ kern_return_t thread_get_state(
  *	Change thread's machine-dependent state.
  */
 kern_return_t thread_set_state(
-	thread_t		thread,
+	register thread_t	thread,
 	int			flavor,
 	thread_state_t		new_state,
 	natural_t		new_state_count)
@@ -1462,7 +1461,7 @@ kern_return_t thread_set_state(
 }
 
 kern_return_t thread_info(
-	thread_t		thread,
+	register thread_t	thread,
 	int			flavor,
 	thread_info_t		thread_info_out,    /* pointer to OUT array */
 	natural_t		*thread_info_count) /*IN/OUT*/
@@ -1474,7 +1473,7 @@ kern_return_t thread_info(
 		return KERN_INVALID_ARGUMENT;
 
 	if (flavor == THREAD_BASIC_INFO) {
-	    thread_basic_info_t	basic_info;
+	    register thread_basic_info_t	basic_info;
 
 	    /* Allow *thread_info_count to be one smaller than the
 	       usual amount, because creation_time is a new member
@@ -1560,7 +1559,7 @@ kern_return_t thread_info(
 	    return KERN_SUCCESS;
 	}
 	else if (flavor == THREAD_SCHED_INFO) {
-	    thread_sched_info_t	sched_info;
+	    register thread_sched_info_t	sched_info;
 
 	    if (*thread_info_count < THREAD_SCHED_INFO_COUNT) {
 		return KERN_INVALID_ARGUMENT;
@@ -1602,7 +1601,7 @@ kern_return_t thread_info(
 }
 
 kern_return_t	thread_abort(
-	thread_t	thread)
+	register thread_t	thread)
 {
 	if (thread == THREAD_NULL || thread == current_thread()) {
 		return KERN_INVALID_ARGUMENT;
@@ -1693,10 +1692,10 @@ thread_t kernel_thread(
  *	This kernel thread runs forever looking for threads to destroy
  *	(when they request that they be destroyed, of course).
  */
-void __attribute__((noreturn)) reaper_thread_continue(void)
+void reaper_thread_continue(void)
 {
 	for (;;) {
-		thread_t thread;
+		register thread_t thread;
 		spl_t s;
 
 		s = splsched();
@@ -1811,12 +1810,12 @@ thread_unfreeze(
 
 void
 thread_doassign(
-	thread_t			thread,
-	processor_set_t			new_pset,
+	register thread_t		thread,
+	register processor_set_t	new_pset,
 	boolean_t			release_freeze)
 {
-	processor_set_t			pset;
-	boolean_t			old_empty, new_empty;
+	register processor_set_t	pset;
+	register boolean_t		old_empty, new_empty;
 	boolean_t			recompute_pri = FALSE;
 	spl_t				s;
 	
@@ -2119,8 +2118,8 @@ thread_policy(
 	int		data)
 {
 #if	MACH_FIXPRI
-	kern_return_t	ret = KERN_SUCCESS;
-	int		temp;
+	register kern_return_t	ret = KERN_SUCCESS;
+	register int	temp;
 	spl_t		s;
 #endif	/* MACH_FIXPRI */
 
@@ -2236,6 +2235,7 @@ thread_wire(
 
 void thread_collect_scan(void)
 {
+#if	0
 	register thread_t	thread, prev_thread;
 	processor_set_t		pset, prev_pset;
 
@@ -2288,6 +2288,7 @@ void thread_collect_scan(void)
 		thread_deallocate(prev_thread);
 	if (prev_pset != PROCESSOR_SET_NULL)
 		pset_deallocate(prev_pset);
+#endif	/* 0 */
 }
 
 boolean_t thread_collect_allowed = TRUE;
@@ -2321,7 +2322,7 @@ void consider_thread_collect(void)
 #if	MACH_DEBUG
 
 vm_size_t stack_usage(
-	vm_offset_t stack)
+	register vm_offset_t stack)
 {
 	int i;
 
@@ -2338,7 +2339,7 @@ vm_size_t stack_usage(
  */
 
 void stack_init(
-	vm_offset_t stack)
+	register vm_offset_t stack)
 {
 	if (stack_check_usage) {
 	    int i;
@@ -2354,7 +2355,7 @@ void stack_init(
  */
 
 void stack_finalize(
-	vm_offset_t stack)
+	register vm_offset_t stack)
 {
 	if (stack_check_usage) {
 	    vm_size_t used = stack_usage(stack);
@@ -2415,7 +2416,7 @@ kern_return_t host_stack_usage(
 	vm_size_t	*maxusagep,
 	vm_offset_t	*maxstackp)
 {
-	natural_t total;
+	unsigned int total;
 	vm_size_t maxusage;
 
 	if (host == HOST_NULL)
@@ -2447,8 +2448,8 @@ kern_return_t processor_set_stack_usage(
 	vm_size_t maxusage;
 	vm_offset_t maxstack;
 
-	thread_t *threads;
-	thread_t tmp_thread;
+	register thread_t *threads;
+	register thread_t tmp_thread;
 
 	unsigned int actual;	/* this many things */
 	unsigned int i;
@@ -2566,7 +2567,7 @@ kern_return_t processor_set_stack_usage(
 void
 thread_stats(void)
 {
-	thread_t thread;
+	register thread_t thread;
 	int total = 0, rpcreply = 0;
 
 	queue_iterate(&default_pset.threads, thread, thread_t, pset_threads) {
